@@ -2,10 +2,20 @@
 
 jQuery(document).ready(function() {
     if(jQuery('.tag-image-wrapper').length) {
+        // Load ajax form
+        jQuery('body').append('<div id="ajax-form" style="display: none; "></div>');
+        jQuery('#ajax-form').load(ImageTags.contextUrl().replace('@@imagetags-manage', '@@imagetags-form-ajax'));
         jQuery('.tag-manage').removeClass('tag-manage').addClass('tag-manage-inline');
         jQuery('#imagetags-form, #imagetags-manage').hide();
         jQuery('#tag-start-tagging').show().click(ImageTags.startTagging);
         jQuery('#tag-stop-tagging').click(ImageTags.stopTagging);
+        // Remove unload protection from forms (taken from formUnload.js)
+        /* Waiting for bug fix in formUnload.js to enable this
+        var tool = window.onbeforeunload && window.onbeforeunload.tool;
+        if (tool) {
+            tool.removeForms.apply(tool, jQuery('form.enableUnloadProtection').get());
+        }
+        */
         // auto-select input fields
         jQuery('.tag-embed-code input').focus(function(event) {event.target.select();});
         ImageTags.prepareEmbedOptions()
@@ -26,9 +36,7 @@ ImageTags = {
     version: '1.0',
     title: 'ImageTags',
     description: 'Client side managing of image tags',
-    hyphen: ' - ',
     draggableConfig: {revert: false, cancel: 'span.tag-link-title'},
-    saveButton: 'Save',
 
     startTagging: function(event) {
         /* When "Start tagging" link is pressed, perform the following actions
@@ -52,17 +60,20 @@ ImageTags = {
         ImageTags.draggableConfig['stop'] = ImageTags.onBoxStopDragging;
         ImageTags.draggableConfig['start'] = ImageTags.onBoxStartDragging;
         ImageTags.turnDraggable(jQuery('.tag-image-wrapper a.tag-link'));
+        // Create helper action links in tag-tagging box
+        ImageTags.createActionLinks(jQuery('.tag-link-plain'));
         // Add a special tag-tagging class to the a.tag-plain links
         jQuery('.tag-tag-list').addClass('tag-tagging');
         // Bind click event to a.tag-edit links
         ImageTags.turnEditable(jQuery('.tag-edit'));
+        // Create a 'confirm remove' box
+        jQuery('body').after('<div style="display: none; " id="imagetags-confirm-remove">' + ImageTagsLabels.removeConfirmText + '</div>');
         // Bind click event to a.tag-remove links
         ImageTags.turnRemovable(jQuery('.tag-remove'));
-        // Re-set ImageTags.hyphen and ImageTags.saveButton values based on markup (it could change according to translations)
-        ImageTags.hyphen = jQuery('#imagetags-hyphen').text();
-        ImageTags.saveButton = form.find('input.submit-widget').hide().addClass('allowMultiSubmit').val()
         // Bind click event to a.tag-link-plain to disable real links
-        ImageTags.preventRealLink(jQuery('.tag-link-plain'));     
+        ImageTags.preventRealLink(jQuery('.tag-link-plain'));
+        // Hide save button and allow multi submit
+        form.find('input.submit-widget').hide().addClass('allowMultiSubmit')
     },
     
     
@@ -90,6 +101,17 @@ ImageTags = {
         ImageTags.linksOverOut();
     },
     
+    contextUrl: function() {
+        /* Calculates the objects URL
+        */
+        var url = document.location.href;
+        if(url.indexOf('?')>-1) {
+            url = url.substring(0, url.indexOf('?'));
+        }
+        return url;
+
+    },
+    
     turnDraggable: function(collection) {
         /* Turn collection into draggable according to ImageTags configuration
          */
@@ -107,6 +129,21 @@ ImageTags = {
          */
         collection.click(ImageTags.onRemoveLinkClick);
     },
+    
+    createActionLinks: function(collection) {
+        var url = ImageTags.contextUrl();
+        collection.each(function(idx, el) {
+            var safeId = el.id.replace('image-tag-link-plain-', '');
+            var id = safeId.replace('-', '.');
+            var buffer = [];
+            buffer.push(' <span class="tag-actions">(');
+            buffer.push('<a class="tag-remove" href="' + url + '?form.widgets.remove:list=' + id + '&ajax:int=1">' + ImageTagsLabels.removeLinkText + '</a>');
+            buffer.push(' | ');
+            buffer.push('<a class="tag-edit" id="image-tag-edit-' + safeId + '" href="' + url + '?id=' + id + '">' + ImageTagsLabels.editLinkText + '</a>');
+            buffer.push(')</span>');
+            jQuery(el).after(buffer.join('\n'));
+        });
+    },
 
     preventRealLink: function(collection){
         /* Prevent click event propagation an default to collection
@@ -227,21 +264,18 @@ ImageTags = {
         event.preventDefault();
         event.stopPropagation();
         // Call to confirm dialog
-        var confirmContainer = jQuery('#imagetags-confirm-remove-wrapper')
-        var confirmDiv = confirmContainer.find('#imagetags-confirm-remove');
+        var confirmDiv = jQuery('#imagetags-confirm-remove');
         
-        var yes = confirmContainer.find('span').first().text();
-        var no = confirmContainer.find('span').last().text()
         var buttons = {}
-        buttons[no] = function() {
+        buttons[ImageTagsLabels.noButtonText] = function() {
             jQuery(this).dialog('close');
         };
-        buttons[yes] = function() {
+        buttons[ImageTagsLabels.yesButtonText] = function() {
             jQuery(this).dialog('close');
-            jQuery.get(event.target.href + '&ajax:int=1', ImageTags.onRemoveSuccess);
+            jQuery.get(event.target.href, ImageTags.onRemoveSuccess);
         };
         confirmDiv.dialog({
-            title: confirmContainer.find('em').text(), 
+            title: ImageTagsLabels.removeLinkText,
             resizable: false,
             buttons: buttons,
             modal: true
@@ -290,8 +324,6 @@ ImageTags = {
         /* Display the edit tag details form
          */
         var formContainer = jQuery('#ajax-form');
-        // Get the title
-        var title = formContainer.find('h2').hide().text();
         // onclose function
         var onclose = function(event, ui) {
             ImageTags.resetForm();
@@ -301,9 +333,9 @@ ImageTags = {
         };
         // Display the form dialog
         buttons = {};
-        buttons[ImageTags.saveButton] = ImageTags.submitForm;
+        buttons[ImageTagsLabels.saveButtonText] = ImageTags.submitForm;
         formContainer.dialog({
-            title: title, 
+            title: ImageTagsLabels.tagFormTitle, 
             resizable: false,
             close: onclose,
             open: function(event, ui) {
@@ -368,12 +400,13 @@ ImageTags = {
             ImageTags.turnDraggable(jQuery('.tag-image-wrapper').append(boxHTML.text()).children().last());
             // Create the new tag-plain link
             plainTags = jQuery('.tag-plain-tags');
-            plainTags.append((plainTags.children().length > 0 ? ImageTags.hyphen : '') + titleHTML.text());
+            plainTags.append((plainTags.children().length > 0 ? ImageTagsLabels.hyphenLabel : '') + titleHTML.text());
             plainTags.show().parent().find('.tag-plain-no-tag').hide();
         }
         // Modify events of newly created elements in the page
         ImageTags.turnDraggable(jQuery('#image-tag-' + safeId));
         ImageTags.preventRealLink(jQuery('#image-tag-link-plain-' + safeId));
+        ImageTags.createActionLinks(jQuery('#image-tag-link-plain-' + safeId));
         ImageTags.turnEditable(jQuery('#image-tag-plain-' + safeId + ' .tag-edit'));
         ImageTags.turnRemovable(jQuery('#image-tag-plain-' + safeId + ' .tag-remove'));
 
@@ -397,7 +430,7 @@ ImageTags = {
         children.each(function(pos, el) {
             plainTags.append(el);
             if(pos<children.length-1) {
-                plainTags.append(ImageTags.hyphen);
+                plainTags.append(ImageTagsLabels.hyphenLabel);
             }
         });
         // If all labels were removed, show "No tags yet" legend.
@@ -445,4 +478,6 @@ for(var fn in ImageTags_bak) {
 }
 
 // and delete the backed up ImageTags
-delete window['ImageTags_bak']
+if(window['ImageTags_bak']) {
+    delete window['ImageTags_bak'];
+}
